!(function (window, document, $, undefined) {
    var Callbackable = function () {
        if (!(this instanceof Callbackable)) {
            return new Callbackable();
        }
        return this;
    };
    Callbackable.prototype = {
        callbackSerial: 0,
        callbacks: {},
        callback: function (callbackId, data) {
            try {
                this.callbacks[callbackId] && this.callbacks[callbackId](data);
                delete this.callbacks[callbackId];
            } catch (e) {
                alert(e);
            }
        },
        registCallback: function (cb) {
            if (cb && typeof cb == 'function') {
                var callbackId = 'TYClient_' + ++this.callbackSerial;
                this.callbacks[callbackId] = cb;
                return callbackId
            } else {
                return null;
            }
        }
    }
    var iOSClient = function () {
        if (!(this instanceof iOSClient)) {
            return new iOSClient();
        }
        return this;
    };
    iOSClient.prototype = Callbackable();
    iOSClient.prototype.call = function (uri, args, cb) {
        if (cb && typeof cb == 'function') {
            var callbackId = this.registCallback(cb);
            args = args || {};
            args['callbackId'] = callbackId;
        }
        var src = uri;
        if (args) {
            src += '?' + $.param(args);
        }
        var iframe = $('<iframe/>').attr('src', src).attr('height', '1px').attr('width', '1px');
        $('body').append(iframe);
        iframe.remove();
        delete iframe;
    };
    iOSClient.prototype.get_version = function () {
        return 1;
    };
    iOSClient.prototype.appinfo = function (cb) {
        return '1';
    };
    iOSClient.prototype.login = function (cb) {
        this.call('typ://login', undefined, cb);
    };
    iOSClient.prototype.favorBook = function (bookId, cb) {
        this.call('typ://book/favor/' + bookId, undefined, cb);
    };
    iOSClient.prototype.unfavorBook = function (bookId, cb) {
        this.call('typ://book/unfavor/' + bookId, undefined, cb);
    };
    iOSClient.prototype.follow = function (userId, cb) {
        this.call('typ://user/follow/' + userId, undefined, cb);
    };
    iOSClient.prototype.unfollow = function (userId, cb) {
        this.call('typ://user/unfollow/' + userId, undefined, cb);
    };
    iOSClient.prototype.sendPumpkin = function (bookId, cb) {
        this.call('typ://book/send/pumpkin/' + bookId, undefined, cb);
    };
    iOSClient.prototype.releaseComment = function (bookId, cb) {
        this.call('typ://comment/release/' + bookId, undefined, cb);
    };
    iOSClient.prototype.snsShare = function (title, content, link, imageUrl, cb) {
        this.call('typ://sns/share', {title: title, content: content, link: link, imageUrl: imageUrl}, cb);
    };
    iOSClient.prototype.snsShareSelectPlatform = function (title, content, link, imageUrl, sharePlatforms, cb) {
        this.call('typ://sns/share', {title: title, content: content, link: link, imageUrl: imageUrl, sharePlatforms: sharePlatforms}, cb);
    };
    iOSClient.prototype.alert = function (message) {
        alert(message);
    };
    var AndroidClient = function (client) {
        if (!(this instanceof AndroidClient)) {
            return new AndroidClient(client);
        }
        this.client = client;
        return this;
    };
    AndroidClient.prototype = Callbackable();
    AndroidClient.prototype.get_version = function () {
        return 1;
    };
    AndroidClient.prototype.appinfo = function (cb) {
        this.client.appinfo(this.registCallback(cb) || '');
    };
    AndroidClient.prototype.login = function (cb) {
        this.client.login(this.registCallback(cb) || '');
    };
    AndroidClient.prototype.favorBook = function (bookId, cb) {
        this.client.favorBook(bookId, this.registCallback(cb) || '');
    };
    AndroidClient.prototype.unfavorBook = function (bookId, cb) {
        this.client.unfavorBook(bookId, this.registCallback(cb) || '');
    };
    AndroidClient.prototype.sendPumpkin = function (bookId, cb) {
        this.client.sendPumpkin(bookId, this.registCallback(cb) || '');
    };
    AndroidClient.prototype.follow = function (userId, cb) {
        this.client.follow(userId, this.registCallback(cb) || '');
    };
    AndroidClient.prototype.unfollow = function (userId, cb) {
        this.client.unfollow(userId, this.registCallback(cb) || '');
    };
    AndroidClient.prototype.releaseComment = function (bookId, cb) {
        this.client.releaseComment(bookId, this.registCallback(cb) || '');
    };
    AndroidClient.prototype.snsShare = function (title, content, link, imageUrl, cb) {
        this.client.snsShare(title, content, link, imageUrl, this.registCallback(cb) || '');
    };
    AndroidClient.prototype.snsShareSelectPlatform = function (title, content, link, imageUrl, sharePlatforms, cb) {
        this.client.snsShare(title, content, link, imageUrl, sharePlatforms, this.registCallback(cb) || '');
    };
    AndroidClient.prototype.alert = function (message) {
        this.client.alert(message);
    };
    var TYClient = function () {
        if (!(this instanceof TYClient)) {
            return new TYClient();
        }
        return this;
    };
    if (window.AndroidNativeClient) {
        TYClient.prototype = AndroidClient(window.AndroidNativeClient);
    } else {
        TYClient.prototype = iOSClient();
    }
    _TYClient = window.TYClient;
    window.TYClient = TYClient();
})(window, document, jQuery);
